/*
 * Copyright 2013-2016 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except
 * in compliance with the License. You may obtain a copy of the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software distributed under the License
 * is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing permissions and limitations under
 * the License.
 */

package ru.ispras.castle.ast.basis;

/**
 * {@link SyntaxNodeVisitor} is a base interface of a {@link SyntaxNode} visitor.
 *
 * @see SyntaxNode
 * 
 * @author <a href="mailto:kamkin@ispras.ru">Alexander Kamkin</a>
 */
public interface SyntaxNodeVisitor<Tag extends Enum<Tag>> {
  /**
   * {@link Result} contains possible visitor results.
   */
  public enum Result {
    /**
     * Continue traversal.
     */
    OK,
    /**
     * Do not traverse the children subtrees. This result can be returned by the
     * {@link SyntaxNodeVisitor#onEnd} handler.
     */
    PRUNE,
    /**
     * Repeat traversal of the node. This result can be returned by the {@link SyntaxNodeVisitor#onEnd}
     * handler.
     */
    REPEAT,
    /**
     * Stop traversal.
     */
    ABORT
  }
  
  /**
   * Called before traversing the child nodes.
   * 
   * @param node the visited node.
   *
   * @return the visiting result.
   */
  Result onBegin(SyntaxNode<Tag> node);

  /**
   * Called when the child nodes have been traversed.
   * 
   * @param node the visited node.
   *
   * @return the visiting result.
   */
  Result onEnd(SyntaxNode<Tag> node);
}
