/*
 * Copyright 2018-2019 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except
 * in compliance with the License. You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License
 * is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing permissions and limitations under
 * the License.
 */

package ru.ispras.castle.ast;

import org.junit.Assert;
import org.junit.Test;

import java.math.BigInteger;
import java.util.LinkedList;
import java.util.List;

/**
 * The test for {@link IntegerRange} class.
 * @author <a href="mailto:smolov@ispras.ru">Sergey Smolov</a>
 */
public class IntegerRangeTestCase {

  @Test
  public void sortRangeNoIntersectTest() {

    final IntegerRange first = new IntegerRange(BigInteger.ZERO, BigInteger.ONE);
    final IntegerRange second = new IntegerRange(BigInteger.valueOf(2), BigInteger.valueOf(2));
    final IntegerRange third = new IntegerRange(BigInteger.valueOf(4), BigInteger.valueOf(4));

    final List<IntegerRange> ranges = new LinkedList<>();
    ranges.add(first);
    ranges.add(third);
    ranges.add(second);

    /* Check that sorting results in an increasing order value. */
    ranges.sort(IntegerRange::compareTo);
    Assert.assertArrayEquals(ranges.toArray(new IntegerRange[0]),
        new IntegerRange[]{first, second, third});
  }

  @Test
  public void sortRangeIntersectTest() {

    final IntegerRange first = new IntegerRange(BigInteger.ZERO, BigInteger.valueOf(4));
    final IntegerRange second = new IntegerRange(BigInteger.valueOf(2), BigInteger.valueOf(2));
    final IntegerRange third = new IntegerRange(BigInteger.valueOf(2), BigInteger.valueOf(4));

    final List<IntegerRange> ranges = new LinkedList<>();
    ranges.add(first);
    ranges.add(third);
    ranges.add(second);

    /* Check that sorting results in an increasing order value. */
    ranges.sort(IntegerRange::compareTo);
    Assert.assertArrayEquals(ranges.toArray(new IntegerRange[0]),
        new IntegerRange[]{first, second, third});
  }
}
